@@ -1,6 +1,6 @@
 /*
  * Copyright © 2009 CNRS
- * Copyright © 2009-2011 inria.  All rights reserved.
+ * Copyright © 2009-2014 Inria.  All rights reserved.
  * Copyright © 2009-2011 Université Bordeaux 1
  * Copyright © 2011 Cisco Systems, Inc.  All rights reserved.
  * Copyright © 2011      Oracle and/or its affiliates.  All rights reserved.
@@ -15,8 +15,12 @@
 
 #include <stdio.h>
 #include <errno.h>
+#ifdef HAVE_DIRENT_H
 #include <dirent.h>
+#endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <sys/types.h>
 #include <sys/processor.h>
 #include <sys/procset.h>
@@ -136,6 +140,7 @@ hwloc_solaris_set_thisthread_cpubind(hwloc_topology_t topology, hwloc_const_bitm
 static int
 hwloc_solaris_get_sth_cpubind(hwloc_topology_t topology, idtype_t idtype, id_t id, hwloc_bitmap_t hwloc_set, int flags __hwloc_attribute_unused)
 {
+  processorid_t binding;
   int depth = hwloc_get_type_depth(topology, HWLOC_OBJ_NODE);
   int n;
   int i;
@@ -145,15 +150,21 @@ hwloc_solaris_get_sth_cpubind(hwloc_topology_t topology, idtype_t idtype, id_t i
     return -1;
   }
 
+  /* first check if processor_bind() was used to bind to a single processor rather than to an lgroup */
+  if ( processor_bind(idtype, id, PBIND_QUERY, &binding) == 0 && binding != PBIND_NONE ) {
+    hwloc_bitmap_only(hwloc_set, binding);
+    return 0;
+  }
+
+  /* if not, check lgroups */
   hwloc_bitmap_zero(hwloc_set);
   n = hwloc_get_nbobjs_by_depth(topology, depth);
-
   for (i = 0; i < n; i++) {
     hwloc_obj_t obj = hwloc_get_obj_by_depth(topology, depth, i);
     lgrp_affinity_t aff = lgrp_affinity_get(idtype, id, obj->os_index);
 
     if (aff == LGRP_AFF_STRONG)
-      hwloc_bitmap_or(hwloc_set, hwloc_set, obj->cpuset);      
+      hwloc_bitmap_or(hwloc_set, hwloc_set, obj->cpuset);
   }
 
   if (hwloc_bitmap_iszero(hwloc_set))
@@ -293,7 +304,7 @@ hwloc_solaris_get_thisthread_membind(hwloc_topology_t topology, hwloc_nodeset_t
 #endif /* HAVE_LIBLGRP */
 
 
-#ifdef MADV_ACCESS_LWP 
+#ifdef MADV_ACCESS_LWP
 static int
 hwloc_solaris_set_area_membind(hwloc_topology_t topology, const void *addr, size_t len, hwloc_const_nodeset_t nodeset, hwloc_membind_policy_t policy, int flags __hwloc_attribute_unused)
 {
@@ -370,7 +381,7 @@ browse(struct hwloc_topology *topology, lgrp_cookie_t cookie, lgrp_id_t lgrp, hw
     obj->memory.page_types_len = 2;
     obj->memory.page_types = malloc(2*sizeof(*obj->memory.page_types));
     memset(obj->memory.page_types, 0, 2*sizeof(*obj->memory.page_types));
-    obj->memory.page_types[0].size = getpagesize();
+    obj->memory.page_types[0].size = hwloc_getpagesize();
 #ifdef HAVE__SC_LARGE_PAGESIZE
     obj->memory.page_types[1].size = sysconf(_SC_LARGE_PAGESIZE);
 #endif
@@ -438,7 +449,6 @@ hwloc_look_lgrp(struct hwloc_topology *topology)
 
 #ifdef HAVE_LIBKSTAT
 #include <kstat.h>
-#define HWLOC_NBMAXCPUS 1024 /* FIXME: drop */
 static int
 hwloc_look_kstat(struct hwloc_topology *topology)
 {
@@ -451,38 +461,48 @@ hwloc_look_kstat(struct hwloc_topology *topology)
   kstat_named_t *stat;
   unsigned look_cores = 1, look_chips = 1;
 
-  unsigned numsockets = 0;
-  unsigned proc_physids[HWLOC_NBMAXCPUS];
-  unsigned proc_osphysids[HWLOC_NBMAXCPUS];
-  unsigned osphysids[HWLOC_NBMAXCPUS];
+  unsigned Pproc_max = 0;
+  unsigned Pproc_alloc = 256;
+  struct hwloc_solaris_Pproc {
+    unsigned Lsock, Psock, Lcore, Lproc;
+  } * Pproc = malloc(Pproc_alloc * sizeof(*Pproc));
 
-  unsigned numcores = 0;
-  unsigned proc_coreids[HWLOC_NBMAXCPUS];
-  unsigned oscoreids[HWLOC_NBMAXCPUS];
+  unsigned Lproc_num = 0;
+  unsigned Lproc_alloc = 256;
+  struct hwloc_solaris_Lproc {
+    unsigned Pproc;
+  } * Lproc = malloc(Lproc_alloc * sizeof(*Lproc));
 
-  unsigned core_osphysids[HWLOC_NBMAXCPUS];
+  unsigned Lcore_num = 0;
+  unsigned Lcore_alloc = 256;
+  struct hwloc_solaris_Lcore {
+    unsigned Pcore, Psock;
+  } * Lcore = malloc(Lcore_alloc * sizeof(*Lcore));
 
-  unsigned numprocs = 0;
-  unsigned proc_procids[HWLOC_NBMAXCPUS];
-  unsigned osprocids[HWLOC_NBMAXCPUS];
+  unsigned Lsock_num = 0;
+  unsigned Lsock_alloc = 256;
+  struct hwloc_solaris_Lsock {
+    unsigned Psock;
+  } * Lsock = malloc(Lsock_alloc * sizeof(*Lsock));
 
-  unsigned physid, coreid, cpuid;
-  unsigned procid_max = 0;
+  unsigned sockid, coreid, cpuid;
   unsigned i;
 
-  for (cpuid = 0; cpuid < HWLOC_NBMAXCPUS; cpuid++)
-    {
-      proc_procids[cpuid] = -1;
-      proc_physids[cpuid] = -1;
-      proc_osphysids[cpuid] = -1;
-      proc_coreids[cpuid] = -1;
-    }
+  for (i = 0; i < Pproc_alloc; i++) {
+    Pproc[i].Lproc = -1;
+    Pproc[i].Lsock = -1;
+    Pproc[i].Psock = -1;
+    Pproc[i].Lcore = -1;
+  }
 
-  if (!kc)
-    {
-      hwloc_debug("kstat_open failed: %s\n", strerror(errno));
-      return 0;
-    }
+  if (!kc) {
+    hwloc_debug("kstat_open failed: %s\n", strerror(errno));
+    free(Pproc);
+    free(Lproc);
+    free(Lcore);
+    free(Lsock);
+    return 0;
+  }
 
   for (ksp = kc->kc_chain; ksp; ksp = ksp->ks_next)
     {
@@ -490,11 +510,6 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	continue;
 
       cpuid = ksp->ks_instance;
-      if (cpuid > HWLOC_NBMAXCPUS)
-	{
-	  fprintf(stderr,"CPU id too big: %u\n", cpuid);
-	  continue;
-	}
 
       if (kstat_read(kc, ksp, NULL) == -1)
 	{
@@ -503,12 +518,28 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	}
 
       hwloc_debug("cpu%u\n", cpuid);
-      proc_procids[cpuid] = numprocs;
-      osprocids[numprocs] = cpuid;
-      numprocs++;
 
-      if (cpuid >= procid_max)
-        procid_max = cpuid + 1;
+      if (cpuid >= Pproc_alloc) {
+	Pproc_alloc *= 2;
+	Pproc = realloc(Pproc, Pproc_alloc * sizeof(*Pproc));
+	for(i = Pproc_alloc/2; i < Pproc_alloc; i++) {
+	  Pproc[i].Lproc = -1;
+	  Pproc[i].Lsock = -1;
+	  Pproc[i].Psock = -1;
+	  Pproc[i].Lcore = -1;
+	}
+      }
+      Pproc[cpuid].Lproc = Lproc_num;
+
+      if (Lproc_num >= Lproc_alloc) {
+	Lproc_alloc *= 2;
+	Lproc = realloc(Lproc, Lproc_alloc * sizeof(*Lproc));
+      }
+      Lproc[Lproc_num].Pproc = cpuid;
+      Lproc_num++;
+
+      if (cpuid >= Pproc_max)
+        Pproc_max = cpuid + 1;
 
       stat = (kstat_named_t *) kstat_data_lookup(ksp, "state");
       if (!stat)
@@ -528,7 +559,7 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	stat = (kstat_named_t *) kstat_data_lookup(ksp, "chip_id");
 	if (!stat)
 	  {
-	    if (numsockets)
+	    if (Lsock_num)
 	      fprintf(stderr, "could not read socket id for CPU%u: %s\n", cpuid, strerror(errno));
 	    else
 	      hwloc_debug("could not read socket id for CPU%u: %s\n", cpuid, strerror(errno));
@@ -537,17 +568,17 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	  }
 	switch (stat->data_type) {
 	  case KSTAT_DATA_INT32:
-	    physid = stat->value.i32;
+	    sockid = stat->value.i32;
 	    break;
 	  case KSTAT_DATA_UINT32:
-	    physid = stat->value.ui32;
+	    sockid = stat->value.ui32;
 	    break;
 #ifdef _INT64_TYPE
 	  case KSTAT_DATA_UINT64:
-	    physid = stat->value.ui64;
+	    sockid = stat->value.ui64;
 	    break;
 	  case KSTAT_DATA_INT64:
-	    physid = stat->value.i64;
+	    sockid = stat->value.i64;
 	    break;
 #endif
 	  default:
@@ -555,14 +586,19 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	    look_chips = 0;
 	    continue;
 	}
-	proc_osphysids[cpuid] = physid;
-	for (i = 0; i < numsockets; i++)
-	  if (physid == osphysids[i])
+	Pproc[cpuid].Psock = sockid;
+	for (i = 0; i < Lsock_num; i++)
+	  if (sockid == Lsock[i].Psock)
 	    break;
-	proc_physids[cpuid] = i;
-	hwloc_debug("%u on socket %u (%u)\n", cpuid, i, physid);
-	if (i == numsockets)
-	  osphysids[numsockets++] = physid;
+	Pproc[cpuid].Lsock = i;
+	hwloc_debug("%u on socket %u (%u)\n", cpuid, i, sockid);
+	if (i == Lsock_num) {
+	  if (Lsock_num == Lsock_alloc) {
+	    Lsock_alloc *= 2;
+	    Lsock = realloc(Lsock, Lsock_alloc * sizeof(*Lsock));
+	  }
+	  Lsock[Lsock_num++].Psock = sockid;
+	}
       } while(0);
 
       if (look_cores) do {
@@ -570,7 +606,7 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	stat = (kstat_named_t *) kstat_data_lookup(ksp, "core_id");
 	if (!stat)
 	  {
-	    if (numcores)
+	    if (Lcore_num)
 	      fprintf(stderr, "could not read core id for CPU%u: %s\n", cpuid, strerror(errno));
 	    else
 	      hwloc_debug("could not read core id for CPU%u: %s\n", cpuid, strerror(errno));
@@ -597,16 +633,19 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 	    look_cores = 0;
 	    continue;
 	}
-	for (i = 0; i < numcores; i++)
-	  if (coreid == oscoreids[i] && proc_osphysids[cpuid] == core_osphysids[i])
+	for (i = 0; i < Lcore_num; i++)
+	  if (coreid == Lcore[i].Pcore && Pproc[cpuid].Psock == Lcore[i].Psock)
 	    break;
-	proc_coreids[cpuid] = i;
+	Pproc[cpuid].Lcore = i;
 	hwloc_debug("%u on core %u (%u)\n", cpuid, i, coreid);
-	if (i == numcores)
-	  {
-	    core_osphysids[numcores] = proc_osphysids[cpuid];
-	    oscoreids[numcores++] = coreid;
+	if (i == Lcore_num) {
+	  if (Lcore_num == Lcore_alloc) {
+	    Lcore_alloc *= 2;
+	    Lcore = realloc(Lcore, Lcore_alloc * sizeof(*Lcore));
 	  }
+	  Lcore[Lcore_num].Psock = Pproc[cpuid].Psock;
+	  Lcore[Lcore_num++].Pcore = coreid;
+	}
       } while(0);
 
       /* Note: there is also clog_id for the Thread ID (not unique) and
@@ -616,72 +655,150 @@ hwloc_look_kstat(struct hwloc_topology *topology)
 
   if (look_chips) {
     struct hwloc_obj *obj;
-    unsigned j;
-    hwloc_debug("%d Sockets\n", numsockets);
-    for (j = 0; j < numsockets; j++) {
-      obj = hwloc_alloc_setup_object(HWLOC_OBJ_SOCKET, osphysids[j]);
+    unsigned j,k;
+    hwloc_debug("%d Sockets\n", Lsock_num);
+    for (j = 0; j < Lsock_num; j++) {
+      obj = hwloc_alloc_setup_object(HWLOC_OBJ_SOCKET, Lsock[j].Psock);
       if (CPUType)
 	hwloc_obj_add_info(obj, "CPUType", CPUType);
       if (CPUModel)
 	hwloc_obj_add_info(obj, "CPUModel", CPUModel);
-      hwloc_object_cpuset_from_array(obj, j, proc_physids, procid_max);
+      obj->cpuset = hwloc_bitmap_alloc();
+      for(k=0; k<Pproc_max; k++)
+	if (Pproc[k].Lsock == j)
+	  hwloc_bitmap_set(obj->cpuset, k);
       hwloc_debug_1arg_bitmap("Socket %d has cpuset %s\n", j, obj->cpuset);
       hwloc_insert_object_by_cpuset(topology, obj);
     }
     hwloc_debug("%s", "\n");
   }
 
-  if (look_cores)
-    hwloc_setup_level(procid_max, numcores, oscoreids, proc_coreids, topology, HWLOC_OBJ_CORE);
-
-  if (numprocs)
-    hwloc_setup_level(procid_max, numprocs, osprocids, proc_procids, topology, HWLOC_OBJ_PU);
+  if (look_cores) {
+    struct hwloc_obj *obj;
+    unsigned j,k;
+    hwloc_debug("%d Cores\n", Lcore_num);
+    for (j = 0; j < Lcore_num; j++) {
+      obj = hwloc_alloc_setup_object(HWLOC_OBJ_CORE, Lcore[j].Pcore);
+      obj->cpuset = hwloc_bitmap_alloc();
+      for(k=0; k<Pproc_max; k++)
+	if (Pproc[k].Lcore == j)
+	  hwloc_bitmap_set(obj->cpuset, k);
+      hwloc_debug_1arg_bitmap("Core %d has cpuset %s\n", j, obj->cpuset);
+      hwloc_insert_object_by_cpuset(topology, obj);
+    }
+    hwloc_debug("%s", "\n");
+  }
+  if (Lproc_num) {
+    struct hwloc_obj *obj;
+    unsigned j,k;
+    hwloc_debug("%d PUs\n", Lproc_num);
+    for (j = 0; j < Lproc_num; j++) {
+      obj = hwloc_alloc_setup_object(HWLOC_OBJ_PU, Lproc[j].Pproc);
+      obj->cpuset = hwloc_bitmap_alloc();
+      for(k=0; k<Pproc_max; k++)
+	if (Pproc[k].Lproc == j)
+	  hwloc_bitmap_set(obj->cpuset, k);
+      hwloc_debug_1arg_bitmap("PU %d has cpuset %s\n", j, obj->cpuset);
+      hwloc_insert_object_by_cpuset(topology, obj);
+    }
+    hwloc_debug("%s", "\n");
+  }
 
   kstat_close(kc);
 
-  return numprocs > 0;
+  free(Pproc);
+  free(Lproc);
+  free(Lcore);
+  free(Lsock);
+
+  return Lproc_num > 0;
 }
 #endif /* LIBKSTAT */
 
-void
-hwloc_look_solaris(struct hwloc_topology *topology)
+static int
+hwloc_look_solaris(struct hwloc_backend *backend)
 {
+  struct hwloc_topology *topology = backend->topology;
   unsigned nbprocs = hwloc_fallback_nbprocessors (topology);
+  int alreadypus = 0;
+
+  if (topology->levels[0][0]->cpuset)
+    /* somebody discovered things */
+    return 0;
+
+  hwloc_alloc_obj_cpusets(topology->levels[0][0]);
+
 #ifdef HAVE_LIBLGRP
   hwloc_look_lgrp(topology);
 #endif /* HAVE_LIBLGRP */
 #ifdef HAVE_LIBKSTAT
   nbprocs = 0;
-  if (hwloc_look_kstat(topology))
-    return;
+  if (hwloc_look_kstat(topology) > 0)
+    alreadypus = 1;
 #endif /* HAVE_LIBKSTAT */
-  hwloc_setup_pu_level(topology, nbprocs);
+  if (!alreadypus)
+    hwloc_setup_pu_level(topology, nbprocs);
 
   hwloc_obj_add_info(topology->levels[0][0], "Backend", "Solaris");
+  if (topology->is_thissystem)
+    hwloc_add_uname_info(topology, NULL);
+  return 1;
 }
 
 void
-hwloc_set_solaris_hooks(struct hwloc_topology *topology)
+hwloc_set_solaris_hooks(struct hwloc_binding_hooks *hooks,
+			struct hwloc_topology_support *support __hwloc_attribute_unused)
 {
-  topology->set_proc_cpubind = hwloc_solaris_set_proc_cpubind;
-  topology->set_thisproc_cpubind = hwloc_solaris_set_thisproc_cpubind;
-  topology->set_thisthread_cpubind = hwloc_solaris_set_thisthread_cpubind;
+  hooks->set_proc_cpubind = hwloc_solaris_set_proc_cpubind;
+  hooks->set_thisproc_cpubind = hwloc_solaris_set_thisproc_cpubind;
+  hooks->set_thisthread_cpubind = hwloc_solaris_set_thisthread_cpubind;
 #ifdef HAVE_LIBLGRP
-  topology->get_proc_cpubind = hwloc_solaris_get_proc_cpubind;
-  topology->get_thisproc_cpubind = hwloc_solaris_get_thisproc_cpubind;
-  topology->get_thisthread_cpubind = hwloc_solaris_get_thisthread_cpubind;
-  topology->set_proc_membind = hwloc_solaris_set_proc_membind;
-  topology->set_thisproc_membind = hwloc_solaris_set_thisproc_membind;
-  topology->set_thisthread_membind = hwloc_solaris_set_thisthread_membind;
-  topology->get_proc_membind = hwloc_solaris_get_proc_membind;
-  topology->get_thisproc_membind = hwloc_solaris_get_thisproc_membind;
-  topology->get_thisthread_membind = hwloc_solaris_get_thisthread_membind;
+  hooks->get_proc_cpubind = hwloc_solaris_get_proc_cpubind;
+  hooks->get_thisproc_cpubind = hwloc_solaris_get_thisproc_cpubind;
+  hooks->get_thisthread_cpubind = hwloc_solaris_get_thisthread_cpubind;
+  hooks->set_proc_membind = hwloc_solaris_set_proc_membind;
+  hooks->set_thisproc_membind = hwloc_solaris_set_thisproc_membind;
+  hooks->set_thisthread_membind = hwloc_solaris_set_thisthread_membind;
+  hooks->get_proc_membind = hwloc_solaris_get_proc_membind;
+  hooks->get_thisproc_membind = hwloc_solaris_get_thisproc_membind;
+  hooks->get_thisthread_membind = hwloc_solaris_get_thisthread_membind;
 #endif /* HAVE_LIBLGRP */
-#ifdef MADV_ACCESS_LWP 
-  topology->set_area_membind = hwloc_solaris_set_area_membind;
-  topology->support.membind->firsttouch_membind = 1;
-  topology->support.membind->bind_membind = 1;
-  topology->support.membind->interleave_membind = 1;
-  topology->support.membind->nexttouch_membind = 1;
+#ifdef MADV_ACCESS_LWP
+  hooks->set_area_membind = hwloc_solaris_set_area_membind;
+  support->membind->firsttouch_membind = 1;
+  support->membind->bind_membind = 1;
+  support->membind->interleave_membind = 1;
+  support->membind->nexttouch_membind = 1;
 #endif
 }
+
+static struct hwloc_backend *
+hwloc_solaris_component_instantiate(struct hwloc_disc_component *component,
+				    const void *_data1 __hwloc_attribute_unused,
+				    const void *_data2 __hwloc_attribute_unused,
+				    const void *_data3 __hwloc_attribute_unused)
+{
+  struct hwloc_backend *backend;
+  backend = hwloc_backend_alloc(component);
+  if (!backend)
+    return NULL;
+  backend->discover = hwloc_look_solaris;
+  return backend;
+}
+
+static struct hwloc_disc_component hwloc_solaris_disc_component = {
+  HWLOC_DISC_COMPONENT_TYPE_CPU,
+  "solaris",
+  HWLOC_DISC_COMPONENT_TYPE_GLOBAL,
+  hwloc_solaris_component_instantiate,
+  50,
+  NULL
+};
+
+const struct hwloc_component hwloc_solaris_component = {
+  HWLOC_COMPONENT_ABI,
+  NULL, NULL,
+  HWLOC_COMPONENT_TYPE_DISC,
+  0,
+  &hwloc_solaris_disc_component
+};
