@@ -1,6 +1,6 @@
 /*
  * Copyright © 2009 CNRS
- * Copyright © 2009-2013 Inria.  All rights reserved.
+ * Copyright © 2009-2014 Inria.  All rights reserved.
  * Copyright © 2009-2011 Université Bordeaux 1
  * Copyright © 2009-2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
@@ -51,7 +51,7 @@ hwloc_xml_callbacks_reset(void)
 {
   hwloc_nolibxml_callbacks = NULL;
   hwloc_libxml_callbacks = NULL;
-}			       
+}
 
 /************************************************
  ********* XML import (common routines) *********
@@ -284,10 +284,10 @@ hwloc__xml_import_object_attr(struct hwloc_topology *topology __hwloc_attribute_
    * deprecated (from 1.0)
    */
   else if (!strcmp(name, "dmi_board_vendor")) {
-    hwloc_obj_add_info(obj, "DMIBoardVendor", strdup(value));
+    hwloc_obj_add_info(obj, "DMIBoardVendor", value);
   }
   else if (!strcmp(name, "dmi_board_name")) {
-    hwloc_obj_add_info(obj, "DMIBoardName", strdup(value));
+    hwloc_obj_add_info(obj, "DMIBoardName", value);
   }
 
   /*************************
@@ -559,8 +559,7 @@ hwloc__xml_import_object(hwloc_topology_t topology,
     if (state->next_attr(state, &attrname, &attrvalue) < 0)
       break;
     if (!strcmp(attrname, "type")) {
-      obj->type = hwloc_obj_type_of_string(attrvalue);
-      if (obj->type == (hwloc_obj_type_t)-1)
+      if (hwloc_obj_type_sscanf(attrvalue, &obj->type, NULL, NULL, 0) < 0)
         return -1;
     } else {
       /* type needed first */
@@ -1406,7 +1405,7 @@ hwloc_export_obj_userdata_base64(void *reserved,
   hwloc__xml_export_state_t state = reserved;
   size_t encoded_length;
   char *encoded_buffer;
-  int ret;
+  int ret __hwloc_attribute_unused;
 
   if (name && hwloc__xml_export_check_buffer(name, strlen(name)) < 0) {
     errno = EINVAL;
@@ -1524,6 +1523,7 @@ static struct hwloc_disc_component hwloc_xml_disc_component = {
 
 const struct hwloc_component hwloc_xml_component = {
   HWLOC_COMPONENT_ABI,
+  NULL, NULL,
   HWLOC_COMPONENT_TYPE_DISC,
   0,
   &hwloc_xml_disc_component
